// Copyright Creative Computing, Morristown, New Jersey
// C conversion by Payton Byrd from BASIC in "More BASIC
// Computer Games" by Creative Computing.

#include <ctype.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <stdbool.h>
//#include <time.h>
//#include <conio.h>
//#include <peekpoke.h>

unsigned char arrows = 5u;

unsigned char items[6];

// Rooms and their connections
const unsigned char rooms[20][3] =
{
	{ 2, 5, 8 },	// 1
	{ 1, 3, 10 },	// 2
	{ 2, 4, 12 },	// 3
	{ 3, 5, 14 },	// 4
	{ 1, 4, 6 },	// 5
	{ 5, 7, 15 },	// 6
	{ 6, 8, 17 },	// 7
	{ 1, 7, 9 },	// 8
	{ 8, 10, 18 },	// 9
	{ 2, 9, 11 },	// 10
	{ 10, 12, 19 },	// 11
	{ 3, 11, 13 },	// 12
	{ 12, 14, 20 }, // 13
	{ 4, 13, 15 },	// 14
	{ 6, 14, 16 },	// 15
	{ 15, 17, 20 },	// 16
	{ 7, 16, 18 },	// 17
	{ 9, 17, 19 },	// 18
	{ 11, 18, 20 },	// 19
	{ 13, 16, 19 }	// 20
};

// Generates a random number between 1 and 20
unsigned char fna()
{
	return (unsigned char)(rand() % 20u + 1u);
}

// Generates a random number between 1 and 3
unsigned char fnb()
{
	return (unsigned char)(rand() % 3u + 1u);
}

// Generates a random number between 1 and 4
unsigned char fnc()
{
	return (unsigned char)(rand() % 4u + 1u);
}

// Ensures all items are in different rooms.
bool validateItems(void)
{
	unsigned char i, j;

	for(i = 0; i < 6; ++i)
	{
		for(j = 0; j < 6; ++j)
		{
			if(i != j)
			{
				if(items[i] == items[j]) 
				{
					// OOPS, two items in
					// same room, no good!
					return false;
				}
			}
		}
	}

	return true;
}

// Place each item in a separate room.
void randomizeItems(void)
{
	// Randomize the world
	// We use clock here 
	// because a human's input
	// to load the game should
	// be variable, even over 
	// small sets.
	//
	// However, launching in an
	// emulator using autostart
	// will always result in the
	// same random seed because
	// there is no variance in
	// the amount of time it takes
	// to get to this point.
	srand(clock() % 32767);

	do
	{
		items[0] = fna(); // You
		items[1] = fna(); // Wumpus
		items[2] = fna(); // Bat
		items[3] = fna(); // Bat
		items[4] = fna(); // Pit
		items[5] = fna(); // Pit
	}
	while(!validateItems());
}

// Displays information about the current room.
void printLocation()
{	
	unsigned char k;

	puts("");

	for(k = 0; k < 3; ++k)
	{
		if(rooms[items[0]-1][k] == items[1])
		{
			puts("I smell a wumpus!");
		}

		if(rooms[items[0]-1][k] == items[2]
			|| rooms[items[0]-1][k] == items[3])
		{
			puts("Bats nearby!");
		}

		if(rooms[items[0]-1][k] == items[4]
			|| rooms[items[0]-1][k] == items[5])
		{
			puts("I feel a draft!");
		}
	}

	printf("You are in room %u\n", items[0]);
	printf("Tunnels lead to %u, %u, %u\n",
		rooms[items[0]-1][0],
		rooms[items[0]-1][1],
		rooms[items[0]-1][2]);
}

// User selects whether to shoot or move.
unsigned char chooseOption()
{
	unsigned char input = '\0';
	//POKE(0x02fc,0xff);

	printf("\nShoot or move [M or S]: ");
	while(input != 'm' && input != 's')
	{
		input = getc(stdin);
		tolower(input);
	}

	return input;
}

// User selects the number of rooms
// to shoot the arrow through.
unsigned char chooseNumberOfRooms()
{
	unsigned char input = 0;
	unsigned char result[1] = "";

	printf("\nNo. of rooms (1-5):");
	while(input < '1' || input > '5')
	{
		input = fgetc(stdin);
	}

	sprintf(result, "%c", input);

	return (unsigned char)atoi(result);
}

// User selects a room adjacent to the current
// room.
unsigned char chooseRoom(unsigned char currentRoom)
{
	char input[3];
	char selectedRoom;
	int result = -1;
	
	printf("\nChoose room [%u, %u, %u]: ",
		rooms[currentRoom - 1][0],
		rooms[currentRoom - 1][1],
		rooms[currentRoom - 1][2]);
	do
	{
			
		fgets(input, sizeof input, stdin);

		selectedRoom = (unsigned char)atoi(input);

		if(selectedRoom == rooms[currentRoom - 1][0]
			|| selectedRoom == rooms[currentRoom - 1][1]
			|| selectedRoom == rooms[currentRoom - 1][2])
		{
			result = selectedRoom;
		}
	} 
	while(result == -1);

	//puts("\n");

	return (unsigned char)result;
}

// User fires an arrow
char shootArrow()
{
	unsigned char counter, room;
	unsigned char numberOfRooms;
	
	// Get the number of rooms for the arrow
	// to travel.
	numberOfRooms = chooseNumberOfRooms();
		
	// Start in our room.
	room = items[0];

	// Loop the number of rooms selected.
	for(counter = 0; counter < numberOfRooms; ++counter)
	{
		// Move arrow to chosen room.
		room = chooseRoom(room);
	}

	// Did we hit him?
	if(room == items[1])
	{
		puts("\nAHA!  You got the Wumpus!");
		puts("The Winner is you!");
		return 1;
	}

	// Did we hit ourself?
	if(room == items[0])
	{
		puts("OUCH!  The arrow got you!");
		return -1;
	}

	// Decrement arrow count
	--arrows;
	puts("Missed.");

	if(arrows == 0)
	{
		// Uh oh.
		puts("You are out of arrows!");
		return -1;
	}

	return 0;
}

// Moves the wumpus.
char moveWumpus()
{
	// Randomly chooses another room.
	items[1] = rooms[items[1]][fnc()];
	
	// Are we sharing a room with the wumpus now?
	if(items[1] == items[0])
	{
		puts("\nTSK TSK TSK - Wumpus got you!");
		return 1;
	}

	return 0;
}

// Move the player
char movePlayer()
{
	unsigned char newRoom = 0;
	char wumpusResult;

	// Choose the room to move to.
	newRoom = chooseRoom(items[0]);

	// Did we actually move?
	if(items[0] == newRoom)
	{
		puts("Can't stay still!");
		return 0;
	}

	// Set the items array to our new 
	// room.
	items[0] = newRoom;

	// Did we find the wumpus?
	if(items[1] == newRoom)
	{
		// Yup, make him move.
		puts("\n... OOPS!  Bumped a Wumpus!");
		
		wumpusResult = moveWumpus();
		
		return wumpusResult;
	}
	// Did we find a bat?
	else if(items[2] == newRoom || items[3] == newRoom)
	{
		// Yup, it's off to Elsewhereville with the player
		puts("\nZAP--Super Bat Snatch!!  \nElsewhereville for you!");

		// Player's position is randomly selected from
		// all 20 rooms.
		items[0] = fna();
	}
	// Did we find a pit?
	else if(items[4] == newRoom || items[5] == newRoom)
	{
		// Dum, dum, de dum...
		puts("\nYYYIIIIEEEE . . . Fell in pit!");
		return 1;
	}

	return 0;
}

// Ye old main routine.
void main(void)
{
	unsigned char option, shootResult;
	for(;;)
	{
		// Put everything in it's place.
		randomizeItems();

		// Polite header.
	#ifdef __CBM__
		putchar(147);
	#endif
		puts("              WUMPUS!");
		puts("         CREATIVE COMPUTING");
		puts("           Morristown, NJ");
		puts("    Converted to C by Payton Byrd");
		puts("   Compiled to Atari by Pirx, 2014");

		// Loop until someone gets hurt.
		for(;;)
		{
			puts("");

			// Display the location to the user.
			printLocation();

			// Get their choice of move or shoot
			option = tolower(chooseOption());

			if(option == 's')
			{
				// They chose to shoot.  Get result.
				shootResult = shootArrow();

				if(shootResult != 0)
				{
					// Something got shot
					break;
				}
			}
			else
			{
				// They chose to move.  Get result.
				if(movePlayer() == 1)
				{
					// Something bad happened to the player.
					break;
				}
			}
		}
		puts("\n\n              GAME OVER");
		puts    ("     Thank you for playing WUMPUS!");
		puts    ("         TRY AGAIN NOW DUDE\n\n");

	}
}

// That's all, folks!
